# Copyright (C) 2014 - 2015 Tobias Rautenkranz
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

.PHONY: images volumes_images label_images
images: volumes_images label_images label_json

#
# Defaults
#
AXIS ?= 2
JPEG_QUALITY ?= 98%
#LO_SCALE ?= 50%
#FLIP_AXIS ?= -1

DATA_DIR ?= ./

LABEL_MAP ?= $(DATA_DIR)/seg.nrrd
LABELS ?= $(firstword $(wildcard $(DATA_DIR)/*.ctbl) \
$(patsubst %.ilf, %.ctbl, $(wildcard $(DATA_DIR)/*.ilf)) \
$(patsubst %.label, %.ctbl,  $(wildcard $(DATA_DIR)/*.label)))

OUT_DIR ?= out
VOLUMES ?= $(filter-out $(LABEL_MAP), $(wildcard $(DATA_DIR)/*.nrrd))

CA_DIR := $(dir $(lastword $(MAKEFILE_LIST)))
CA_UTILS := $(CA_DIR)/util

VOLUMES_TARGETS = $(addprefix $(OUT_DIR)/mq/, $(notdir $(VOLUMES:.nrrd=)))

volumes_images: $(VOLUMES_TARGETS)

label_images: $(OUT_DIR)/mq/lb
label_json: $(OUT_DIR)/labels_colors.json


debug:
	echo $(VOLUMES)

# convert itksnap label description file
%.ctbl: %.label
	$(CA_DIR)/itksnap2ctbl.sh $< > $@

# untested
%.ctbl: %.ilf
	$(CA_DIR)/ilf2ctbl.sh $< > $@



define setup-output-dir
	@rm -rf $@
	@mkdir -p $@
endef

check_volumes: $(LABEL_MAP) $(VOLUMES)
	$(CA_UTILS)/check_volumes.sh  $^


#
# Convert Volume
#
.PRECIOUS: $(OUT_DIR)/hq/%
$(OUT_DIR)/hq/%: $(DATA_DIR)/%.nrrd
	$(setup-output-dir)
	$(CA_UTILS)/stripmetadata.sh $< | teem-unu quantize -b 16 | teem-unu dice -a $(AXIS) -o $@/ 2>&1 | \
		xargs -L 1 echo -ne "\r" # do not spam console with status of every slice
	@echo ""

$(OUT_DIR)/mq/%: $(OUT_DIR)/hq/%
	$(setup-output-dir)
	for i in $</*.png; do \
		convert -strip -quality $(JPEG_QUALITY) $$i $@/`basename $$i png`jpeg; \
	done
	-jpegoptim -q --strip-all	$@/*.jpeg

#
# LQ
#

#$(OUT_DIR)/lq/%: $(OUT_DIR)/hq/%
#	$(setup-output-dir)
#	for i in $</*.png; do \
#		convert -strip -quality $(JPEG_QUALITY) -resize $(LQ_SCALE) $$i $@/`basename $$i png`jpeg; \
#	done
#	-jpegoptim -q --strip-all	$@/*.jpeg

#
# Convert labels
#
$(OUT_DIR)/hq/lb $(OUT_DIR)/labels_colors.json $(OUT_DIR)/mq/lb: $(LABEL_MAP) $(LABELS)
	@rm -rf $(OUT_DIR)/hq/lb $(OUT_DIR)/mq/lb
	$(CA_DIR)/mk_labels.sh -a $(AXIS) $(LABEL_MAP) $(LABELS) $(OUT_DIR)

ifdef NAME
.PHONY: dist
dist: ${NAME}.tar.gz

${NAME}.tar.gz: ${DATA_DIR}/*.nrrd $(wildcard ${DATA_DIR}/*.txt) $(wildcard ${DATA_DIR}/*.label) ${LABELS}
	tar --transform 's,^${DATA_DIR}/,${NAME}/,' -cvzf $@ $^

.PHONY: clean
clean:
	rm -rf $(OUT_DIR) ${NAME}.tar.gz
endif
